@@ -873,7 +873,7 @@ static int parse_vl_num(char *attr, char *field, int *vl)
 
 static int do_perf_counter_set(FILE *f, char *line)
 {
-	char *s = line, *orig, *sp, *nodeid, *attr, *field, *field_trim, *val_error;
+	char *s = line, *orig, *sp, *nodeid, *attr, *field, *field_trim, *end_ptr;
 	Node *node;
 	int portnum, vl;
 	uint64_t value;
@@ -897,13 +897,15 @@ static int do_perf_counter_set(FILE *f, char *line)
 		return -1;
 	}
 
-	if (sp) {
-		portnum = strtoul(sp, 0, 0);
-		if (portnum < 1 || portnum > node->numports) {
-			fprintf(f, "# bad port number %d at nodeid \"%s\"\n",
-				portnum, nodeid);
-			return -1;
-		}
+	if (sp == NULL) {
+		fprintf(f, "# port number not found in command\n");
+		return -1;
+	}
+
+	portnum = strtoul(sp, &end_ptr, 0);
+	if (end_ptr == sp) {
+		fprintf(f, "# port number not found in command\n");
+		return -1;
 	}
 
 	if (!(p = node_get_port(node, portnum))) {
@@ -911,9 +913,11 @@ static int do_perf_counter_set(FILE *f, char *line)
 		return -1;
 	}
 
-	if (s && *s)
-		while (isspace(*s))
-			s++;
+	while (*s != '\0' && !isspace(*s))
+		s++;
+	while (*s != '\0'  && isspace(*s))
+		s++;
+
 	attr = strsep(&s, ".");
 	if(s == NULL)
 		goto format_error;
@@ -934,14 +938,33 @@ static int do_perf_counter_set(FILE *f, char *line)
 		field_trim--;
 	*(field_trim + 1) = 0;
 
+#ifndef ULLONG_MAX
+/* Maximum value an `unsigned long long int' can hold. (Minimum is 0.) */
+#   define ULLONG_MAX   18446744073709551615ULL
+#endif
+
 	errno = 0;
-	value = strtoull(s, &val_error, 0);
+	value = strtoull(s, &end_ptr, 0);
 	if((value == 0 || value == ULLONG_MAX) && errno != 0) {
-		fprintf(f, "# value is not valid integer\n");
+		fprintf(f, "# value '%s' is out of range [%d,%llu]\n", s, 0, ULLONG_MAX);
 		return -1;
 	}
-	if(*val_error) {
-		fprintf(f, "# value %s is not valid integer\n", s);
+	if(end_ptr == s  || end_ptr == NULL || *end_ptr !='\0') {
+		fprintf(f, "# value '%s' is not valid integer\n", s);
+		return -1;
+	}
+
+	if (portnum < 0 || portnum > node->numports
+	    || (node->type != SWITCH_NODE && portnum == 0)) {
+		fprintf(f, "# bad port number %d at nodeid \"%s\"\n",
+		        portnum, nodeid);
+		return -1;
+	}
+
+	if (node->type == SWITCH_NODE
+	    && portnum == 0
+	    && !mad_get_field(node->sw->switchinfo, 0, IB_SW_ENHANCED_PORT0_F)) {
+		fprintf(f, "# Can't set performance counters of base switch port 0\n");
 		return -1;
 	}
 
@@ -1089,7 +1112,7 @@ static int do_perf_counter_set(FILE *f, char *line)
 		fprintf(f, "# attribute %s not found\n", attr);
 		return -1;
 	}
-	fprintf(f, "%s.%s has been set to %"PRIu64"\n", attr, field, value);
+	fprintf(f, "%s[%d] %s.%s has been set to %"PRIu64"\n", node->nodeid, p->portnum, attr, field, value);
 	return 0;
 field_not_found:
 	fprintf(f, "# field %s cannot be found in attribute %s\n", field, attr);
